////////////////////////////////////////////////////////////////////////
// Class:       HighETrigger
// Module Type: filter
// File:        HighETrigger_module.cc
//
// Generated at Mon Dec 17 15:29:53 2012 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"	
#include "art/Framework/Principal/Event.h"
#include "DDTUtilities/AssociationUtil.h"

#include "DDTBaseDataProducts/CompareDAQHit.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

//#include "RawData/RawTrigger.h"

#include <string>
#include <vector>
#include <algorithm>

namespace novaddt {
  class HighETrigger;
}

class novaddt::HighETrigger : public art::EDFilter {
public:
  explicit HighETrigger(fhicl::ParameterSet const & p);
  virtual ~HighETrigger();

  virtual bool filter(art::Event & e) override;

private:
  bool send_trigger(unsigned const sum_adc, 
		    unsigned const n_hits_total) const;

  std::string slice_label_, slice_instance_;
  unsigned prescale_, trigger_counts_;
  unsigned sum_adc_trigger_threshold_;
  unsigned n_hits_trigger_threshold_;
};

novaddt::HighETrigger::HighETrigger(fhicl::ParameterSet const & p) :
  slice_label_(p.get<std::string>("slice_label")),
  slice_instance_(p.get<std::string>("slice_instance")),
  prescale_(p.get<unsigned>("prescale")),
  trigger_counts_(0),
  sum_adc_trigger_threshold_
  (p.get<unsigned>("sum_adc_trigger_threshold")),
  n_hits_trigger_threshold_
  (p.get<unsigned>("n_hits_trigger_threshold"))
{
  produces<std::vector<TriggerDecision>>();
}

novaddt::HighETrigger::~HighETrigger()
{
  std::cout << "\n\tNumber of High Energy Triggers = " << trigger_counts_
  	    << "\n" << std::endl;
}

bool novaddt::HighETrigger::filter(art::Event & e)	
{
  bool result = false;

  std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);

  art::Handle<std::vector<novaddt::HitList> > slices;
  e.getByLabel(slice_label_, slice_instance_, slices);

  // We need to subtract off the trigger time, so that we know what 
  // time stamp in the EVD this corresponds to:
  // art::Handle<std::vector<rawdata::RawTrigger>> RawTriggers;
  // e.getByLabel("daq", RawTriggers);
  // assert(RawTriggers->size() == 1);
  // TDC event_time(RawTriggers->at(0).fTriggerTimingMarker_TimeStart);

  for (auto const& slice : *slices)
  {
    unsigned sum_adc = 0;

    for (auto const& hit : slice)
      sum_adc += hit.ADC().val;

    double n_hits_total = slice.size();
    
    if (send_trigger(sum_adc, n_hits_total))
    {
      ++trigger_counts_;
      if (trigger_counts_ % prescale_ == prescale_ - 1) 
      {
	auto time_interval = 
	  std::minmax_element(slice.begin(), slice.end(), CompareDAQHit<TDC>());
	auto t0 = time_interval.first->TDC();
	auto t1 = time_interval.second->TDC();
	auto dt = t1 - t0;
	
	// double tdc_to_us = 1.0 / 64e6 * 1e6;
	// std::cout << "\nRun    " << e.run()
	// 	  << "\nSubrun " << e.subRun()
	// 	  << "\nEvent  " << e.id().event()
	// 	  << "\nt0 =   " << (t0 - event_time) * tdc_to_us
	// 	  << "\nt1 =   " << (t1 - event_time) * tdc_to_us
	// 	  << "\ndt =   " << dt
	// 	  << std::endl;

	trigger_decisions->emplace_back
	  (t0, dt, daqdataformats::trigID::TRIG_ID_DATA_ENERGY, prescale_);

	result = true;
      }
    }
  }

  e.put(std::move(trigger_decisions));
  return result;
}

bool novaddt::HighETrigger::send_trigger
(unsigned const sum_adc,
 unsigned const n_hits_total) const
{
  if (sum_adc < sum_adc_trigger_threshold_)
    return false;

  if (n_hits_total < n_hits_trigger_threshold_)
    return false;

  return true;
}

DEFINE_ART_MODULE(novaddt::HighETrigger)
